@@ -1,53 +1,18 @@
-# Copyright (c) 2005-2024 Broadcom. All Rights Reserved.
-# The term "Broadcom" refers to Broadcom Inc. and/or its subsidiaries.
-
-import sys
-import importlib
+# Copyright (c) 2005-2025 Broadcom. All Rights Reserved.
+# Broadcom Confidential. The term "Broadcom" refers to Broadcom Inc.
+# and/or its subsidiaries.
 
 version_info = (
-    8,
+    9,
+    0,
     0,
-    3,
     0,
-    1,
 )
 
 version_info_str = ".".join(str(v) for v in version_info)
 
-if sys.version_info < (2, 7, 9):
-    sys.stderr.write("pyVmomi requires Python 2.7.9 or newer")
-    sys.exit(1)
-
-_initialized = False
-
-
-# Deprecated
-# Definition precedes pyVmomi modules imports to escape circular
-# dependency error if modules import _assert_not_initialized()
-def _assert_not_initialized():
-    if _initialized:
-        raise RuntimeError("pyVmomi is already initialized!")
-
-
-# Deprecated
-try:
-    settings = importlib.import_module('.pyVmomiSettings', 'pyVmomi')
-except ImportError:
-    settings = None
-
-# Deprecated
-# set default settings
-_allowGetSet = getattr(settings, 'allowGetSet', True)
-_allowCapitalizedNames = \
-    getattr(settings, 'allowCapitalizedNames', True)
-_binaryIsBytearray = \
-    getattr(settings, 'binaryIsBytearray', False)
-_legacyThumbprintException = \
-    getattr(settings, 'legacyThumbprintException', False)
-
-
+from . import _init_utils  # noqa: E402
 from . import VmomiSupport  # noqa: E402
-from . import Feature  # noqa: E402
 from ._typeinfos import load_typeinfos  # noqa: E402
 load_typeinfos()
 
@@ -59,8 +24,23 @@ VmomiSupport.GetVmodlType("vmodl.DynamicData")
 
 from .SoapAdapter import (  # noqa: E402,F401
     SessionOrientedStub, SoapStubAdapter, StubAdapterBase)
-if _legacyThumbprintException:
-    from .Security import ThumbprintMismatchException  # noqa: F401
+
+# Deprecated
+# ThumbprintMismatchException was originally part of SoapAdapter and
+# not in a separate Security module. The insertion into SoapAdapter
+# and the top-level pyvmomi module was for backwards compatibility and
+# is deprecated since pyVmomi 8.0.3
+class ThumbprintMismatchExceptionIsNoMore:
+    def __init__(self, *args, **kwargs):
+        raise Exception("pyVmomi.ThumbprintMismatchException and"
+                        " pyVmomi.SoapAdapter.ThumbprintMismatchException"
+                        " are deprecated."
+                        " Use pyVmomi.Security.ThumbprintMismatchException"
+                        " instead.")
+
+ThumbprintMismatchException = ThumbprintMismatchExceptionIsNoMore
+setattr(SoapAdapter, 'ThumbprintMismatchException',
+        ThumbprintMismatchExceptionIsNoMore)
 
 types = VmomiSupport.types
 
@@ -78,20 +58,14 @@ CreateManagedType = VmomiSupport.CreateManagedType
 # NOT ALLOWED: from pyVmomi import vim.host
 _globals = globals()
 for name in VmomiSupport._topLevelNames:
+    # TODO: remove when support for upper-case module names is dropped
     upperCaseName = VmomiSupport.Capitalize(name)
     obj = VmomiSupport.LazyModule(name)
     _globals[name] = obj
-    if _allowCapitalizedNames:
+    if VmomiSupport._allowCapitalizedNames:
         _globals[upperCaseName] = obj
     if not hasattr(VmomiSupport.types, name):
         setattr(VmomiSupport.types, name, obj)
-        if _allowCapitalizedNames:
+        if VmomiSupport._allowCapitalizedNames:
             setattr(VmomiSupport.types, upperCaseName, obj)
 del _globals
-
-# Deprecated
-def init():
-    _assert_not_initialized()
-    Feature._init()
-    global _initialized
-    _initialized = True
