@@ -1,8 +1,6 @@
-# Copyright (c) 2005-2024 Broadcom. All Rights Reserved.
+# Copyright (c) 2005-2025 Broadcom. All Rights Reserved.
 # The term "Broadcom" refers to Broadcom Inc. and/or its subsidiaries.
 
-import sys
-
 
 def load_typeinfos():
     try:
@@ -40,20 +38,11 @@ def load_typeinfos():
     except ImportError:
         pass
 
+    from .VmomiSupport import SetFreezeDefinitions  # noqa: E402
     try:
-        VmomiSupport.SetFreezeDefinitions(True)
+        SetFreezeDefinitions(True)
         from . import _typeinfo_vsanhealth
     except ImportError:
         pass
     finally:
-        VmomiSupport.SetFreezeDefinitions(False)
-
-
-# Deprecated
-# VmomiJSONEncoder was originally part of VmomiSupport and not a separate module.
-# This insertion into VmomiSupport is for backwards compatibility.
-from .VmomiJSONEncoder import VmomiJSONEncoder
-from .VmomiJSONEncoder import templateOf
-VmomiSupport = sys.modules['pyVmomi.VmomiSupport']
-setattr(VmomiSupport, 'VmomiJSONEncoder', VmomiJSONEncoder)
-setattr(VmomiSupport, 'templateOf', templateOf)
+        SetFreezeDefinitions(False)
