@@ -1,5 +1,7 @@
-# VMware vSphere Python SDK
-# Copyright (c) 2008-2015 VMware, Inc. All Rights Reserved.
+# VMware vSphere Python SDK tests
+#
+# Copyright (c) 2008-2024 Broadcom. All Rights Reserved.
+# The term "Broadcom" refers to Broadcom Inc. and/or its subsidiaries.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -12,6 +14,7 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
+
 from __future__ import print_function
 
 import tests
@@ -23,7 +26,7 @@ class ManagedObjectTests(tests.VCRTestBase):
 
     @tests.VCRTestBase.my_vcr.use_cassette('root_folder_parent.yaml',
                       cassette_library_dir=tests.fixtures_path,
-                      record_mode='once')
+                      record_mode='once', decode_compressed_response=True)
     def test_root_folder_parent(self):
         # see: http://python3porting.com/noconv.html
         si = connect.SmartConnect(host='vcsa',
